/* Copyright (C) 2001  Martin J. Muench <muench@gmc-online.de> */

#include "config.h"

extern void sendpacket(char *, char *);
extern void start_curses();
extern void end_curses();
extern void enigma(char *, int);
extern void pwsetup(char *);

void  nickcat ( char *dest, char *src, int len );
void  check_pw( char *password );

int main (int argc, char **argv) {
 int numActiveFDs, maxFD, null=0, auth=0;
 char host[256], *payload, *stext, text[CHARPERLINE], *enc_pw;
 char nick[MAXNICKLEN + 1], hostnick[MAXNICKLEN + 1];
 u_char buf[1024];
 struct hostent	*hostent;
 FILE *fp=stdin;
 fd_set readset;

if (argc < 3) {
 
 /* Ok, give a little help */
  if(argv[1] && !strcmp(argv[1], "-h"))

/******** Help ***********/
	fprintf(stderr, "
icmpchat %s\n--------------\n
Usage: %s <host> <icmp_code> <name>\n
<host>\t\tHostname to connect to
<icmp_code>\tICMP-Code to use
<name>\t\tSet name (default: ''user'')\n
Available ICMP-Codes:\n
 [0] Echo Reply\t\t\t [8] Echo Request
 [5] Redirect\t
 [9] Router advertisement\t[10] Router solicitation
[13] Timestamp request\t\t[14] Timestamp reply
[15] Information request\t[16] Information reply
[17] AddressMask request\n[18] AdressMask reply
\n", VERSION, argv[0]);
/*************************/

  else 
/******** Usage ***********/
	fprintf(stderr, "
icmpchat %s
\nUsage: %s <host> <icmp_code> <name>
       %s -h  for help
\n", VERSION, argv[0], argv[0]);\
/**************************/
exit(1);
}
 /* ICMP_Code */
 icmpcode=atoi(argv[2]); 

 /* Nick=Argument2 if given, else Nick=Hostname */
 nick[0] = 0;
 if(!argv[3]) 	nickcat(nick, "user", MAXNICKLEN); 
 else 		nickcat(nick, argv[3], MAXNICKLEN);

 /* Lookup hostname */
 hostent = Gethostbyname(argv[1]);
 strncpy(host,inet_ntoa(*(struct in_addr*)hostent->h_addr_list[0]),256);

 /* Get the password */
 enc_pw = Getpass("Enter Password for encryption: ");
 
 /* Set the password */
 pwsetup(enc_pw);
 
 /* Open Raw socket and set socket options */
 rawsock = get_raw_sock();
   
 /* Start the fancy Interface */
 start_curses();
  
 /* Print out welcome screen */
 wprintw(output, "\nICMP-Chat %s by Martin J. Muench <mjm@codito.de>\n---------------------------------------------------------\n", VERSION);
 wprintw(output, "
::::> *******************
::::> Welcome to ICMP-Chat
::::> --------------------
::::>
::::> For the latest Version of ICMP Chat take a look at: www.codito.de
::::> To see a list of features just type '/help'
::::> 
::::> Send bug reports, feedbacks and flames to mjm@codito.de
::::> *******************************************************\n
::::> connecting to %s...
", host);
 wrefresh(output);

 /* Return to user privileges */
 setuid(getuid());
 
 maxFD = rawsock;
 FD_ZERO(&readset);

 /* Send first authentication packet */
 sendpacket(PING, host);

 for(;;) {
 
    /* Set FDs */
    FD_SET(rawsock, &readset);
    FD_SET(0, &readset);

    /* Select FDs */
    numActiveFDs = Select((maxFD + 1), &readset);
	
    /* Data on rawsock ? */
    if (FD_ISSET(rawsock, &readset)) {

	/* Receive the data */
        if (recvfrom(rawsock,buf,1024,0,(struct sockaddr *)&saddr,&null) < 0) 
	  break;

	  /* Set structure to detect the payload */
	  ip=(struct ip *)buf;
	  icmp=(struct icmp *)&buf[ip->ip_hl<<2];
		
	  /* Is the source of the package the defined? */
	  if (!strcmp(host,inet_ntoa(ip->ip_src))) {
		  
	    /* Set the payload */
	    payload = (buf + sizeof(struct ip) + sizeof(struct icmp));

	    /* Decrypt Payload */
	    enigma(payload, strlen(payload));

	    /* If SessionID is correct, analyze the data */
	    if(!strncmp(enicrypt, SID, strlen(SID))) {
			
		/* Strip text */
		stext=(enicrypt + strlen(SID));
				
		/* Does payload contain authentication string 1? */
		if(!strcmp(stext,PING) && !auth) {
	
		  /* Send authentication string 1 */
		  sendpacket(PONG, host); auth=1; 

                  /* Send the nickname, very nasty */
                  strncpy(text, "/hnick ", 8);
                  strncat(text, nick, strlen(nick));
                  sendpacket(text, host);
                  
	    }
	
	    /* Does payload contain authentication string 2? */
	    else if(!strcmp(stext,PONG) && (!auth || auth==1)) {
	
		/* First authentication message? */
		if (!auth) {
		  auth=2;
		  sendpacket(PONG, host);
		  
		  /* Send the nickname, very nasty */
		  strncpy(text, "/hnick ", 8);
		  strncat(text, nick, strlen(nick));
		  sendpacket(text, host);
		}

	    }
		
	    /* Check for special paket */
	    else if(stext[0] == '/') {
	    
		    if(!strcmp(stext,"/quit")) {
			wprintw(output, "::::> %s has left the chat\n\n", hostnick);
			wrefresh(output);
		    }

		    else if(strstr(stext, "/hnick")) {
			nickcat(hostnick, (stext + 7), MAXNICKLEN);
		    	wprintw(output, "::::> %s is connected\n\n", hostnick);
		    	wrefresh(output);
		    }
		    
		    else if(strstr(stext, "/setname")) {
                  	nickcat(hostnick, (stext + 9), MAXNICKLEN);
		    	wprintw(output, "::::> %s is now known as %s\n", hostnick, stext+9);
		   	wrefresh(output);
		    }
	    }

	    /* Authentication done, chat */
	    else if (auth) {	
		wprintw(output, "%s: %s\n", hostnick, stext);
		wrefresh(output);
	    }
	  }
      }
    }   
	
    /* Data on stdin? */
    else if (FD_ISSET(fileno(fp), &readset)) { 
    if (wgetnstr(input, text, CHARPERLINE - 1)) exit;
      else {

	/* The professional ;-) quit-command */
 	if(text[0] == '/') {
 	
		/* Quit icmpchat */
	 	if(!strcmp(text,"/quit")) { 
		  sendpacket("/quit",host);	
		  end_curses(); close(rawsock); exit(1); 
		}
	
		/* Need Help? */
		else if(!strcmp(text,"/help")) {
			wprintw(output, "::::> ******* Help *******
::::> Available Commands:
::::> -------------------
::::> /code x\t\t- Change icmpcode to 'x'
::::>\t\t\t  (see man icmpchat(8) or icmpchat -h for options)
::::> /setname x\t- Change name to x
::::> /about\t\t- Show informations about icmpchat
::::> /help\t\t- Display this message
::::> /quit\t\t- Quit icmpchat the friendly way
::::> **************************************************\n");
		wrefresh(output);
		}
		
		/* Show about */   
		else if(!strcmp(text,"/about")) {
			wprintw(output, "::::> *** About ***
::::> ICMP-Chat %s
::::> Get it at: www.codito.de
::::> ************************\n", VERSION);
			wrefresh(output);
		}
		
		/* Change icmpcode */
		else if(strstr(text, "/code")) {
			icmpcode=get_code(text);
			wprintw(output, "::::> icmpcode set to: %d\n", icmpcode);
			wrefresh(output);
		}

		/* Change nickname */
		else if(strstr(text, "/setname")) {
		  sendpacket(text, host);
		  nickcat(nick, (text + 9), MAXNICKLEN);
		  wprintw(output, "::::> You are now known as %s\n", nick);
		  wrefresh(output);
		}
		
		/* Invalid command */
		else {
			wprintw(output, "::::> unknown command: %s\n", text);
			wrefresh(output);
		}

	}

	else {
	  /* Print text */
	  wprintw(output, "%s\n", text);
	  wrefresh(output);
	
	
      	   /* Send text */
	  sendpacket(text, host);
			
	}
		
	/* Clear Input */
	werase(input);  
	wmove(input, 0, 0);
	wrefresh(input);
    }
  }
 }
exit(0);
}

void  nickcat (char *dest, char *src, int len) {
  int nlen;

  nlen = strlen(src);
  dest[0] = 0;
  strncat(dest,  src, nlen > len ? len : nlen);
}

int get_code(char *string) {
	char *pos = strstr(string, "code");
	return atoi(pos + 5);
}

